;
; vgmplayer.asm - MTX VGM player
;
; To Do
; 1. VGM's tend to be recorded at 60hz however using VDP interrupt runs at
; 50hz. Ideally use CTC chn 1 running at 60hz instead.
;
;------------------------------------------------------------------------------
; Init player
;------------------------------------------------------------------------------
		ld hl,vgmdata + $40	; Start of VGM, skipping header.
		ld (vgmpos),hl
		ld hl, 735		; VGM Delay (60hz)
		ld (vgmdly),hl
;------------------------------------------------------------------------------
; Init interrupts
;------------------------------------------------------------------------------
		di
		im 2
		ld b,2			; Turn all interrupts off
		ld a,3
lp0		out (8), a
		out (9), a
		out (10), a
		out (11), a
		djnz lp0
;
		ld a, ijtabl >> 8	; Setup interrupt vector table
		ld i, a			; Using CTC chn 0 which is linked to
		ld a, ijtabl & $ff	; the VDP Interrupt.
		out (8), a	
		ld hl, inthndl
		ld (ijtabl), hl
;
		ld a, $a5		; 1 - Enable Interrupts
					; 0 - Select timer mode
					; 1 - Prescaler of 256
					; 0 - Falled edge
					; 0 - Auto trigger when const loaded
					; 1 - Timer Const follows
					; 0 - Continued Operation
					; 1 - Control word
		out (8), a
		ld a, $14		; Timer constant of 20.
		out (8), a
		in a, (2)		; Reset VDP interrupt bit
		ei			; Enable interrupts
jp0		jp jp0			; Infinite loop
;------------------------------------------------------------------------------
; Interrupt Handler. Called every 1/50 second.
;------------------------------------------------------------------------------
inthndl		di			; Turn interrupts off
		push hl			; Save registers
		push bc
		push de
		push af
		in a, (2)		; Check if VDP int set and reset it.
		bit 7, a
		jr z, exit
;
		ld hl,(vgmdly)		; Check if delay has gone past
		ld de,735
		and $ff
		sbc hl,de
		ld (vgmdly),hl
		jr nz, exit
		ld hl,(vgmpos)		; Start processing VGM commands
next		ld a, (hl)
		inc hl
		ld (vgmpos), hl
		cp $66			; Restart VGM cmd
		jr nz, next1
		ld hl,vgmdata + $40	; Start of VGM, skipping header.
		ld (vgmpos),hl
		ld hl, 735		; VGM Delay
		ld (vgmdly),hl
		jr exit
next1		cp $4f			; Game Gear SN76489 stereo. Ignored
		jr nz, psg
		inc hl
		jr next
psg		cp $50			; Write byte to SN76489.
		jr nz, waitnn
		ld a, (hl)
		inc hl
		out (6), a
		in a, (3)
		jr next
waitnn		cp $61			; Wait nn samples
		jr nz, wait60
		ld a, (hl)
		inc hl
		ld d, (hl)
		inc hl
		ld (vgmpos), hl
		ld l, a
		ld h, d
		ld (vgmdly), hl
		jr exit
wait60		cp $62			; Wait 735 samples (60Hz)
		jr nz, exit
		ld (vgmpos),hl
		ld hl, 735
		ld (vgmdly), hl
exit		pop af			; Restore registers
		pop de
		pop bc
		pop hl
		ei			; Return to main code
		reti
;------------------------------------------------------------------------------
; Variables
;------------------------------------------------------------------------------
		nop
		nop
		nop
ijtabl		.DW 0		; Interrupt vector table. Needs to by 
		.DW 0		; 8 byte aligned.
		.DW 0
		.DW 0
vgmpos		.DW 0
vgmdly		.DW 0
;------------------------------------------------------------------------------
; VGM data
;------------------------------------------------------------------------------
vgmdata		.incbin VGMFILE
